/*******************************************************************************
*
*      "cs4281_wrapper-22.c" --  Cirrus Logic-Crystal CS4281 linux audio driver.
*
*      Copyright (C) 2000,2001  Cirrus Logic Corp.  
*            -- tom woller (twoller@crystal.cirrus.com) or
*               (pcaudio@crystal.cirrus.com).
*
*      This program is free software; you can redistribute it and/or modify
*      it under the terms of the GNU General Public License as published by
*      the Free Software Foundation; either version 2 of the License, or
*      (at your option) any later version.
*
*      This program is distributed in the hope that it will be useful,
*      but WITHOUT ANY WARRANTY; without even the implied warranty of
*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*      GNU General Public License for more details.
*
*      You should have received a copy of the GNU General Public License
*      along with this program; if not, write to the Free Software
*      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* 12/20/00 trw - new file. 
*
*******************************************************************************/

#include "cs4281_wrapper.h"
#include <asm/page.h>
#include <asm/io.h>
#include <asm/spinlock.h>

#if LINUX_VERSION_CODE < 0x020213 /* 2.2.19 kernel is more like 2.4.0 */
#define set_current_state(x) current->state=x
#define __set_current_state(x) set_current_state(x)
#endif
#define cs4x_pgoff(vma) ((vma)->vm_offset >> PAGE_SHIFT)
#define virt_to_page(addr) (mem_map + MAP_NR(addr))
#define cs4x_mem_map_reserve(page) mem_map_reserve((page) - mem_map)
#define cs4x_mem_map_unreserve(page) mem_map_unreserve((page) - mem_map)
#define free_dmabuf(state, dmabuf) \
	free_pages((unsigned long)(dmabuf)->rawbuf, (dmabuf)->buforder);
#define free_dmabuf2(state, dmabuf) \
	free_pages((unsigned long)(state)->tmpbuff, (state)->buforder_tmpbuff);

#define list_for_each(pos, head) \
        for (pos = (head)->next; pos != (head); pos = pos->next)

LIST_HEAD(cs_pci_drivers);

struct pci_driver_mapping {
        struct pci_dev *dev;
        struct pci_driver *drv;
        void *driver_data;
};

#define PCI_MAX_MAPPINGS 16
static struct pci_driver_mapping drvmap [PCI_MAX_MAPPINGS] = { { NULL, } , };

void * cs_pci_get_driver_data (struct pci_dev *dev)
{
        int i;
        
        for (i = 0; i < PCI_MAX_MAPPINGS; i++)
                if (drvmap[i].dev == dev)
                        return drvmap[i].driver_data;
        
        return NULL;
}

void cs_pci_set_driver_data (struct pci_dev *dev, void *driver_data)
{
        int i;
        
	CS_DBGOUT(CS_INIT | CS_FUNCTION, 4, 
		printk( "cs4281: cs_pci_set_driver_data()\n"));
        for (i = 0; i < PCI_MAX_MAPPINGS; i++)
                if (drvmap[i].dev == dev) {
                        drvmap[i].driver_data = driver_data;
                        return;
                } 
}

const struct pci_device_id *
cs_pci_match_device(const struct pci_device_id *ids, struct pci_dev *dev)
{
        u16 subsystem_vendor, subsystem_device;
	CS_DBGOUT(CS_INIT | CS_FUNCTION, 4, 
		printk( "cs4281: cs_pci_match_device()+\n"));

        pci_read_config_word(dev, PCI_SUBSYSTEM_VENDOR_ID, &subsystem_vendor);
        pci_read_config_word(dev, PCI_SUBSYSTEM_ID, &subsystem_device);
	CS_DBGOUT(CS_INIT | CS_FUNCTION, 4, 
		printk( "cs4281: cs_pci_match_device() sv=0x%x sd=0x%x\n",
			(unsigned)subsystem_vendor,(unsigned)subsystem_device));

        while (ids->vendor || ids->subvendor || ids->class_mask) {
                if ((ids->vendor == PCI_ANY_ID || ids->vendor == dev->vendor) &&
                    (ids->device == PCI_ANY_ID || ids->device == dev->device) &&
                    (ids->subvendor == PCI_ANY_ID || ids->subvendor == subsystem_vendor) &&
                    (ids->subdevice == PCI_ANY_ID || ids->subdevice == subsystem_device) &&
                    !((ids->class ^ dev->class) & ids->class_mask))
		{
			CS_DBGOUT(CS_INIT | CS_FUNCTION, 4, 
				printk( "cs4281: cs_pci_match_device()- ids=0x%x\n",(unsigned)ids));
                        return ids;
		}
                ids++;
        }
	CS_DBGOUT(CS_INIT | CS_FUNCTION, 4, 
		printk( "cs4281: cs_pci_match_device()- no device found\n"));
        return NULL;
}

static int
pci_announce_device(struct pci_driver *drv, struct pci_dev *dev)
{
        const struct pci_device_id *id;
	int found, i;

	CS_DBGOUT(CS_INIT | CS_FUNCTION, 4, 
		printk( "cs4281: cs_pci_announce_device()+\n"));
        if (drv->id_table) {
                id = cs_pci_match_device(drv->id_table, dev);
                if (!id)
                        return 0;
        } else
                id = NULL;

	found = 0;
	for (i = 0; i < PCI_MAX_MAPPINGS && !found; i++)
		if (!drvmap[i].dev) {
			drvmap[i].dev = dev;
			drvmap[i].drv = drv;
			found = 1;
		}

        if (drv->probe(dev, id) >= 0) { 
		if(found)
			return 1;
	} else {
               drvmap[i - 1].dev = NULL; 
        }
	CS_DBGOUT(CS_INIT | CS_FUNCTION, 4, 
		printk( "cs4281: cs_pci_announce_device()- 0\n"));
        return 0;
}

int
cs_pci_register_driver(struct pci_driver *drv)
{
        struct pci_dev *dev;
        int count = 0, found, i;
	CS_DBGOUT(CS_INIT | CS_FUNCTION, 4, 
		printk( "cs4281: cs_pci_register_driver()+\n"));
#ifdef CONFIG_PCI
        list_add_tail(&drv->node, &cs_pci_drivers);
        pci_for_each_dev(dev) {
                found = 0;
                for (i = 0; i < PCI_MAX_MAPPINGS && !found; i++)
                        if (drvmap[i].dev == dev)
                                found = 1;
                if (!found)
                        count += pci_announce_device(drv, dev);
        }
#endif
	CS_DBGOUT(CS_INIT | CS_FUNCTION, 4, 
		printk( "cs4281: cs_pci_register_driver()- (%d)\n",count));
        return count;
}

void
cs_pci_unregister_driver(struct pci_driver *drv)
{
        struct pci_dev *dev;
        int i, found;
#ifdef CONFIG_PCI
        list_del(&drv->node);
        pci_for_each_dev(dev) {
                found = 0;
                for (i = 0; i < PCI_MAX_MAPPINGS && !found; i++)
                        if (drvmap[i].dev == dev)
                                found = 1;
                if (found) {
                        if (drv->remove)
                                drv->remove(dev);
                        drvmap[i].dev = NULL;
                }
        }
#endif        
}

int cs_pci_enable_device(struct pci_dev *dev)
{
        return 0;
}
void cs4281_null(struct pci_dev *pcidev) { return; }

