/*******************************************************************************
*
*      "cs4281pm.c" --  Cirrus Logic-Crystal CS4281 linux audio driver.
*
*      Copyright (C) 2000,2001  Cirrus Logic Corp.  
*            -- tom woller (twoller@crystal.cirrus.com) or
*               (pcaudio@crystal.cirrus.com).
*
*      This program is free software; you can redistribute it and/or modify
*      it under the terms of the GNU General Public License as published by
*      the Free Software Foundation; either version 2 of the License, or
*      (at your option) any later version.
*
*      This program is distributed in the hope that it will be useful,
*      but WITHOUT ANY WARRANTY; without even the implied warranty of
*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*      GNU General Public License for more details.
*
*      You should have received a copy of the GNU General Public License
*      along with this program; if not, write to the Free Software
*      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* 12/22/00 trw - new file. 
*
*******************************************************************************/

#ifndef NOT_CS4281_PM
#include <config/apm.h>
#include <linux/apm_bios.h>

int cs4281_suspend(struct cs4281_state *s);
int cs4281_resume(struct cs4281_state *s);
#define CS4281_SUSPEND_TBL cs4281_suspend_tbl
#define CS4281_RESUME_TBL cs4281_resume_tbl
#define cs4281_pm_callback cs4281_apm_callback

int cs4281_apm_callback(apm_event_t ae) 
{
	struct cs4281_state *s=NULL;
	struct list_head *entry;

	CS_DBGOUT(CS_PM | CS_FUNCTION, 2, printk(KERN_INFO 
		"cs4281: cs4281_apm_callback(): apm_event=0x%x\n",ae));

	list_for_each(entry, &cs4281_devs)
	{
		s = list_entry(entry, struct cs4281_state, list);
		if (entry == &cs4281_devs)
			return 0;
		if (!s) {
			CS_DBGOUT(CS_PM | CS_ERROR, 2, printk(KERN_INFO
				"cs4281: cs4281_apm_callback(): Error - unable to find audio state struct\n"));
			return -ENODEV;
		}
		VALIDATE_STATE(s);
		switch(ae) {

			case APM_SYS_SUSPEND: 
			case APM_CRITICAL_SUSPEND: 
			case APM_USER_SUSPEND: 
				cs4281_suspend(s);
				break;
			case APM_NORMAL_RESUME: 
			case APM_CRITICAL_RESUME: 
			case APM_STANDBY_RESUME: 
				cs4281_resume(s);
				break;
			case APM_POWER_STATUS_CHANGE:
				CS_DBGOUT(CS_PM, 2, printk(KERN_INFO
					"cs4281: cs4281_apm_callback(): APM_POWER_STATUS_CHANGE \n"));
				break;
			default: 
				CS_DBGOUT(CS_PM, 2, printk(KERN_INFO
					"cs4281: cs4281_apm_callback(): unknown apm_event (0x%x)\n",ae));
				break;
		}
	}
	return 0;
}

struct pm_dev
{
	void		*data;
};
struct pm_dev cs_pmdev;

#ifdef CONFIG_APM
#define cs_pm_register(a, b, c) (struct pm_dev *)&cs_pmdev; apm_register_callback((c));
#define cs_pm_unregister_all(a) apm_unregister_callback((cs4281_pm_callback));
#else
#define cs_pm_register(a, b, c) (struct pm_dev *)&cs_pmdev;
#define cs_pm_unregister_all(a)
#endif
static void cs4281_suspend_tbl(struct pci_dev *pcidev)
{
	struct cs4281_state *s = PCI_GET_DRIVER_DATA(pcidev);
	cs4281_suspend(s);
	return;
}

static void cs4281_resume_tbl(struct pci_dev *pcidev)
{
	struct cs4281_state *s = PCI_GET_DRIVER_DATA(pcidev);
	cs4281_resume(s);
	return;
}

#else
#define CS4281_SUSPEND_TBL cs4281_null
#define CS4281_RESUME_TBL cs4281_null
#endif /* CS4281_PM */

